package data.scripts;

import com.fs.starfarer.api.BaseModPlugin;
import com.fs.starfarer.api.Global;
import data.scripts.campaign.AIW_CampaignManager;
import data.scripts.world.AIW_SectorGen;
import data.scripts.world.AIW_VengeanceGeneratorManager;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import org.dark.shaders.light.LightData;
import org.dark.shaders.util.ShaderLib;
import org.dark.shaders.util.TextureData;

public class AIWarModPlugin extends BaseModPlugin
{
    public static final boolean hasTwigLib;
    public static final boolean hasExerelin;
    
    public static boolean templarsExists;

    static {
        boolean foundTwigLib;
        try {
            Global.getSettings().getScriptClassLoader().loadClass("db.twiglib.TwigUtils");
            foundTwigLib = true;
        } catch (ClassNotFoundException ex) {
            foundTwigLib = false;
        }
        hasTwigLib = foundTwigLib;
        
        boolean foundExerelin;
        try {
            Global.getSettings().getScriptClassLoader().loadClass("data.scripts.world.ExerelinGen");
            foundExerelin = true;
        } catch (ClassNotFoundException ex) {
            foundExerelin = false;
        }
        hasExerelin = foundExerelin;
        
        try {
            Global.getSettings().getScriptClassLoader().loadClass("data.scripts.TEMModPlugin");
            templarsExists = true;
        } catch (ClassNotFoundException ex) {
        }
    }
    
    @Override
    public void beforeGameSave()
    {
    }

    @Override
    public void onGameLoad(boolean newGame)
    {
    }

    @Override
    public void onNewGame()
    {
        if (!hasExerelin)
        {
            new AIW_SectorGen().generate(Global.getSector());
        }
        else
        {
            try {
                Class<?> def = Global.getSettings().getScriptClassLoader().loadClass("exerelin.campaign.SectorManager");
                Method method;
                method = def.getMethod("getCorvusMode");
                Object result = method.invoke(def);
                if ((boolean)result == true)
                {
                    new AIW_SectorGen().generate(Global.getSector());
                }
            } catch (NoSuchMethodException | SecurityException | IllegalAccessException | IllegalArgumentException |
                     InvocationTargetException | ClassNotFoundException ex) {
                // check failed, do nothing
            }
        }
    }
    
    @Override
    public void onApplicationLoad() {  
        ShaderLib.init();  
        LightData.readLightDataCSV("data/lights/aiw_lights.csv");  
        TextureData.readTextureDataCSV("data/lights/aiw_textures.csv");  
    }  

    @Override
    public void onNewGameAfterEconomyLoad()
    {
        boolean useRandom = hasExerelin;
        try {
            Class<?> def = Global.getSettings().getScriptClassLoader().loadClass("exerelin.campaign.SectorManager");
            Method method;
            method = def.getMethod("getCorvusMode");
            Object result = method.invoke(def);
            if ((boolean)result == true)
            {
                useRandom = false;
            }
        } catch (NoSuchMethodException | SecurityException | IllegalAccessException | IllegalArgumentException |
                 InvocationTargetException | ClassNotFoundException ex) {
            // check failed, do nothing
        }
        
        AIW_VengeanceGeneratorManager vengenManager = new AIW_VengeanceGeneratorManager();
        
        //Global.getSector().addListener(vengenManager);    //no need, already registered once on creation or something?
        vengenManager.generate(Global.getSector(), useRandom);
    }
    
    @Override
    public void onEnabled(boolean wasEnabledBefore) {
        if (!wasEnabledBefore) new AIW_CampaignManager();
    }
}